# $Id: Request.pm,v 1.3 2006/12/29 19:50:59 koba Exp $
#	Copyright(c) 2006 Satoshi Kobayashi, All rights reserved.

package XiuMai::HTTP::Request;

use strict;
use XiuMai;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use XiuMai::Auth;
use XiuMai::Util qw(url_encode url_decode);
use vars qw($VERSION);

$VERSION = $XiuMai::VERSION;

sub _qvalue_list {
	my ($str) = @_;
	return ()	if (! defined $str);
	return map { s/;.*$//; $_ } split(/,\s*/, $str);
}

sub new {
	my $class = shift;
	return bless { CGI => new CGI }, $class;
}

sub param {
	my $self = shift;
	return $self->{CGI}->param(@_);
}

sub method {
	my $self = shift;
	return $ENV{REQUEST_METHOD};
}

sub scheme {
	my $self = shift;
	return $ENV{HTTPS} ? 'https' : 'http';
}

sub host {
	my $self = shift;
	return $ENV{HTTP_HOST}
		|| (($ENV{SERVER_PORT} == 80)
				? $ENV{SERVER_NAME}
				: "$ENV{SERVER_NAME}:$ENV{SERVER_PORT}");
}

sub url {
	my $self = shift;
	my %param = @_;
	my $url = $ENV{REQUEST_URI};
	$url =~ s/\?.*//;
	return $url;
}

sub url_param {
	my $self = shift;
	return $self->{CGI}->url_param(@_);
}

sub path_info {
	my $self = shift;
	return $ENV{PATH_INFO} || '';
}

sub script_name {
	my $self = shift;
	return $ENV{SCRIPT_NAME};
}

sub accept_language {
	my $self = shift;
	return _qvalue_list($ENV{HTTP_ACCEPT_LANGUAGE});
}

sub xiumai_base_url {
	my $self = shift;
	return $self->{xiumai_base_url}		if (exists $self->{xiuami_base_url});
	my $url = url_decode($self->url);
	$self->{xiumai_base_url}
			= ($url =~ /^$ENV{SCRIPT_NAME}/) ? $ENV{SCRIPT_NAME} : '';
}

sub xiumai_path_info {
	my $self = shift;
	return $self->{xiumai_path_info}	if (exists $self->{xiuami_path_info});
	my $url = url_decode($self->url);
	$self->{xiumai_path_info}
			= ($url =~ /^$ENV{SCRIPT_NAME}/) ? $ENV{PATH_INFO} : $url;
}

sub xiumai_user {
	my $self = shift;
	return $self->{xiumai_user}	if (exists $self->{xiuami_user});
	$self->{xiumai_user}
		= XiuMai::Auth::get_login_name($self->{CGI}->cookie('XIUMAI_AUTH'));
}

1;
