# $Id: Core.pm,v 1.1 2007/01/06 07:39:58 koba Exp $
#	Copyright(c) 2007 Satoshi Kobayashi, All rights reserved.

package XiuMai::Module::Core;

use strict;
use XiuMai::Util qw(cdata);
use vars qw($VERSION);

$VERSION = do { my @r = (q$Revision: 1.1 $ =~ /\d+/g);
				sprintf("%d." . "%02d" x $#r, @r) };

my @common = qw(class img clear);
my @inline = qw(color size br);
my @block  = ();

my %inline = map { $_, 1 } (@common, @inline);
my %block  = map { $_, 1 } (@common, @block);

sub new {
	my $class = shift;
	return bless {
				inline => \%inline,
				block  => \%block,
			}, $class;
}

sub dispatch {
	my $self = shift;
	my ($mode, $name, $param, $value) = @_;
	return $self->$name($mode, $param, $value)
						if (exists $self->{$mode}->{$name});
}

sub img {
	my $self = shift;
	my ($mode, $param) = @_;
	my ($src, $alt, @option) = split(/,\s*/, $param);

	$src = cdata($src);
	$alt = cdata($alt);

	my (%style, $nolink);
	foreach (@option) {
		if    ($_ eq 'nolink')			{ $nolink = 1; 				}
		elsif (/^(left|right)$/)		{ $style{float}   = $1;		}
		elsif (/^(top|middle|bottom)$/) { $style{'vertical-align'}
														  = $1;		}
		elsif (/^b(\d+)$/)				{ $style{border}  = "$1px";	}
		elsif (/^w(\d+)$/)				{ $style{width}   = "$1px";	}
		elsif (/^h(\d+)$/)				{ $style{height}  = "$1px";	}
		elsif (/^(\d+)x(\d+)$/)			{ $style{width}   = "$1px";
										  $style{height}  = "$2px";	}
		elsif (/^(.+):\s*(.*)$/)		{ $style{$1}      =  $2;	}
	}
	my $style;
	foreach (keys %style) {
		$style .= ' '	if (defined $style);
		$style .= cdata("$_: $style{$_};");
	}
	$style = defined $style ? qq(style="$style") : '';

	my $html = qq(<img src="$src" alt="$alt" $style/>);
	$html = qq(<a href="$src">$html</a>)	if (! $nolink);
	if ($mode eq 'block') {
		$html = qq(<div>$html</div>\n);
	}
	return $html;
}

sub color {
	my $self = shift;
	my (undef, $param, $value) = @_;
	return if (! defined $param || ! defined $value);
	my ($color, $bgcolor) = split(/,\s*/, $param);
	return if (! defined $color);
	my $style = cdata("color: $color;");
	$style .= cdata(" background-color: $bgcolor;")	if (defined $bgcolor);
	return qq(<span style="$style">$value</span>);
}

sub size {
	my $self = shift;
	my (undef, $param, $value) = @_;
	return if (! defined $param || ! defined $value);
	return if ($param !~ /^\d+$/);
	my $size = cdata($param);
	return qq(<span style="font-size: ${size}px;">$value</span>);
}

sub br {
	my $self = shift;
	return qq(<br />);
}

sub clear {
	my $self = shift;
	my ($mode, $param) = @_;
	my $clear = (defined $param && $param =~ /^(left|right)$/) ? $1 : 'both';
	return $mode eq 'inline'
				? qq(<br style="clear: $clear;" />)
				: qq(<div style="clear: $clear;"></div>\n);
}

sub class {
	my $self = shift;
	my ($mode, $param, $value) = @_;
	$param = cdata($param);
	return $mode eq 'block'
			? qq(<div class="$param">$value</div>\n)
			: qq(<span class="$param">$value</span>);
}

1;
