# $Id: Resource.pm,v 1.5 2006/12/29 19:46:33 koba Exp $
#	Copyright(c) 2006 Satoshi Kobayashi, All rights reserved.

package XiuMai::Resource;

use strict;
use XiuMai;
use XiuMai::Resource::Folder;
use XiuMai::Resource::Text;
use XiuMai::Resource::XiuMai;
use XiuMai::XHTML::Resource;
use XiuMai::MIMEType;
use XiuMai::Util qw(canonpath cdata url_encode);
use FileHandle;
use vars qw($VERSION);

$VERSION = $XiuMai::VERSION;

sub new {
	my $class = shift;
	my ($req, $filename) = @_;
	my $self = {};

	$self->{Request} = $req;

	$self->{path} = "$XiuMai::HOME/data" . canonpath($req->path_info);
	if (defined $filename) {
		$self->{path} =~ s|/[^/]+$|/|;
		$self->{path} .= $filename;
		$self->{path} = canonpath($self->{path});
	}
	$self->{path} =~ m|^(.*)$|		and $self->{path} = $1;

	($self->{dirname}, $self->{filename})
							= $self->{path} =~ m|^(.*/)([^/]+)?|;

	$self->{url} = $req->url;
	$self->{url} .= url_encode($filename)	if (defined $filename);
	$self->{url} = canonpath($self->{url});

	-d $self->{path}	and return new XiuMai::Resource::Folder($self);
	-f $self->{path}	or return;

	my $type = XiuMai::MIMEType::mime_type($self->{filename});
	$type eq 'text/x-xiumai'	and return new XiuMai::Resource::XiuMai($self);
	$type =~ m|^text/| 			and return new XiuMai::Resource::Text($self);


	return bless $self, $class;
}

sub request {
	my $self = shift;
	return $self->{Request};
}

sub filename {
	my $self = shift;
	return $self->{filename};
}

sub url {
	my $self = shift;
	return $self->{url};
}

sub type {
	my $self = shift;
	return $self->{type}		if (exists $self->{type});
	return $self->{type} = XiuMai::MIMEType::mime_type($self->{filename});
}

sub size {
	my $self = shift;
	return $self->{size}		if (exists $self->{size});
	return $self->{size} = -s $self->{path};
}

sub mtime {
	my $self = shift;
	return $self->{mtime}		if (exists $self->{mtime});
	return $self->{mtime} = (stat($self->{path}))[9];
}

sub lang {
	my $self = shift;
	return @{$self->{lang}}		if (defined $self->{lang});
}

sub r_ok {
	my $self = shift;
	return 1;
}

sub w_ok {
	my $self = shift;
	return defined $self->request->xiumai_user;
}

sub _cmd {
	my $self = shift;
	return defined $self->{Request}->param('cmd')
					? $self->{Request}->param('cmd') : '';
}

sub open {
	my $self = shift;
	$self->_open			or return;
	return $self->r_ok && $self->_info		if ($self->_cmd eq 'info');
	return $self->w_ok && $self->_edit		if ($self->_cmd eq 'edit');
	return $self->_html;
}

sub _open {
	my $self = shift;
	$self->{fh} = new FileHandle($self->{path})	or return;
	return $self;
}

sub _html {
	my $self = shift;
	return $self;
}

sub _edit {
	my $self = shift;

	my $title = cdata($self->{filename});
	my $url   = cdata($self->url);

	my $html = new XiuMai::XHTML::Resource($self);
	$self->{html}
		= $html->title($self->{filename})
			   ->accept_language($self->request->accept_language)
			   ->as_string(
					qq(<h1><a href="$url">$title</a></h1>\n),
					$html->url_navi,
					$html->upload_form,
				);

	$self->{type}  = undef;
	$self->{size}  = length($self->{html});
	$self->{mtime} = undef;
	$self->{lang}  = [$html->content_language];

	return $self;
}

sub _info {
	my $self = shift;

	my $title = cdata($self->{filename});
	my $url   = cdata($self->url);

	my $html = new XiuMai::XHTML::Resource($self);
	$self->{html}
		= $html->title($self->{filename})
			   ->accept_language($self->request->accept_language)
			   ->as_string(
					qq(<h1><a href="$url">$title</a></h1>\n),
					$html->url_navi,
					$html->dump($self),
				);

	$self->{type}  = undef;
	$self->{size}  = length($self->{html});
	$self->{mtime} = undef;
	$self->{lang}  = [$html->content_language];

	return $self;
}

sub print {
	my $self = shift;

	if (exists $self->{html}) {
		print $self->{html};
		return;
	}

	if (exists $self->{text}) {
		print $self->{text};
		return;
	}

	my $buf;
	while ($self->{fh}->read($buf, 1024*1024*10)) {
		print $buf;
	}
}

sub update {
	my $self = shift;

	return if (! $self->w_ok);

	my $file = $self->{Request}->param('file');

	$self->{redirect_code} = 303;
	$self->{redirect_url}  = './';

	if (! defined $file || ! length $file) {
		unlink($self->{path})	or return;
		return $self;
	}

	my $fh = new FileHandle(">$self->{path}")	or return;
	my $buf;
	while (read($file, $buf, 1024*1024)) {
		print $fh $buf;
	}
	$fh->close;
	return $self;
}

sub redirect {
	my $self = shift;
	return ($self->{redirect_code}, $self->{redirect_url});
}

1;
