# $Id: Text.pm,v 1.13 2007/01/06 12:06:17 koba Exp $
#	Copyright(c) 2006,2007 Satoshi Kobayashi, All rights reserved.

package XiuMai::Text;

use strict;
use XiuMai::Module;
use XiuMai::Util qw(cdata);
use vars qw($VERSION);

$VERSION = do { my @r = (q$Revision: 1.13 $ =~ /\d+/g);
				sprintf("%d." . "%02d" x $#r, @r) };

my $link_regex = qr{
		(?:https?|ftp)://[\w\.\-\/\~\?=%&#:;,\$\+]+
	|	(?:mailto:)?[\w\.\-\?\/]+@[\w\-\.]+
}x;

my $module_regex;
if (5.008 <= $]) {
	$module_regex
		= qr/
				&\w+
				(?: \( (?>[^\)]*) \) )?
				(?: { (?: (??{$module_regex}) | . )*? } )?
				;
			/x;
}
else {
	my $m1		  = qr/&\w+(?:\((?>[^\)]*)\))?(?:{.*?})?;/;
	my $m2		  = qr/&\w+(?:\((?>[^\)]*)\))?(?:{(?:$m1|.)*?})?;/;
	my $m3		  = qr/&\w+(?:\((?>[^\)]*)\))?(?:{(?:$m2|.)*?})?;/;
	my $m4		  = qr/&\w+(?:\((?>[^\)]*)\))?(?:{(?:$m3|.)*?})?;/;
	$module_regex = qr/&\w+(?:\((?>[^\)]*)\))?(?:{(?:$m4|.)*?})?;/;
}

sub new {
	my $class = shift;
	return bless {
				eof		=> [],
				toc		=> [],
				module	=> new XiuMai::Module,
			}, $class;
}

sub parse {
	my $self = shift;
	my ($src) = @_;
	my @src = split(/\x0d\x0a|\x0d|\x0a/, $src);
	$self->{SRC} = \@src;
	return $self->block;
}

sub eof {
	my $self = shift;
	return unshift(@{$self->{eof}}, shift)		if (@_);
	return @{$self->{SRC}} == 0
			|| @{$self->{eof}} && $self->{SRC}->[0] eq $self->{eof}->[0];
}

sub reset {
	my $self = shift;
	shift @{$self->{SRC}}	if (@{$self->{SRC}});
	shift @{$self->{eof}}	if (@{$self->{eof}});
}

sub read_line {
	my $self = shift;
	return ! $self->eof && shift @{$self->{SRC}};
}

sub next_line {
	my $self = shift;
	return ! $self->eof && $self->{SRC}->[0];
}

sub block {
	my $self = shift;
	my ($eof) = @_;
	$self->eof($eof)	if (defined $eof);
	my $html = '';
	while (! $self->eof) {
		my $line = $self->read_line;
		next if ($line =~ /^$/);

		if    ($line =~ /^\!/)		{ $html .= $self->title	($line); 		}
		elsif ($line =~ /^\*/)		{ $html .= $self->h		($line, 1);		}
		elsif ($line =~ /^\-+$/)	{ $html .= $self->hr	($line); 		}
		elsif ($line =~ /^\s/)		{ $html .= $self->pre	($line);		}
		elsif ($line =~ /^\-/)		{ $html .= $self->li	($line, 'ul');	}
		elsif ($line =~ /^\+/)		{ $html .= $self->li	($line, 'ol');	}
		elsif ($line =~ /^\:/)		{ $html .= $self->dl	($line);		}
		elsif ($line =~ /^\|/)		{ $html .= $self->table	($line);		}
		elsif ($line =~ /^\#/)		{ $html .= $self->block_module($line);	}
		elsif ($line eq '>>')		{ $html .= $self->blockquote;			}
		elsif ($line eq '>|')		{ $html .= $self->block_pre;			}
		elsif ($line eq '>||')		{ $html .= $self->np_block('||<');		}
		elsif ($line =~ /^\/\//)	{ 										}
		elsif ($line eq '/*')		{ 		   $self->np_block('*/');		}
		else						{ $html .= $self->p		($line); 		}
	}
	$self->reset		if (defined $eof);
	return $html;
}

sub np_block {
	my $self = shift;
	my ($eof) = @_;
	$self->eof($eof)	if (defined $eof);
	my $html = '';
	while (! $self->eof) {
		$html .= cdata($self->read_line).qq(\n);
	}
	$self->reset		if (defined $eof);
	return $html;
}

sub title {
	my $self = shift;
	return $self->{title}	if (! @_);
	my ($line) = @_;
	$line =~ s/^!\s*//;
	$self->{title} = $line;
	return qq(\n<h1>).$self->line($line,1).qq(</h1>\n);
}

sub p {
	my $self = shift;
	my ($line) = @_;

	my $align = '';
	if    ($line =~ s/^<//)	{ $align = qq( style="text-align: left;");		}
	elsif ($line =~ s/^=//)	{ $align = qq( style="text-align: center;");	}
	elsif ($line =~ s/^>//)	{ $align = qq( style="text-align: right;");		}
	else					{ $line =~ s/^~//;								}
	my $html = qq(<p$align>).$self->line($line);
	while (! $self->eof) {
		$line = $self->next_line;
		last if ($line =~ /^$/);
		last if ($line =~ /^\/\*$/);
		last if ($line =~ /^\/\//);
		last if ($line =~ /^[\~\<\=\>\!\*\s\-\+\:\|\#]/);
		$line = $self->read_line;

		$html .= $self->line($line);
	}
	chomp $html;
	return $html.qq(</p>\n);
}

sub h {
	my $self = shift;
	my ($line, $base_level) = @_;

	$line =~ s/^(\*{1,5})\s*//;
	my $level = length($1);

	my $id;
	$line =~ s/\s*\[([^\[\]]*)\]$//	and $id = $1;

	my $toc = $self->{toc};
	defined $id or $id = sprintf("SEC.%d", scalar @$toc + 1);
	push(@$toc, ('-' x $level).qq([[$line>#$id]]));

	$level += $base_level;
	$id = cdata($id);
	return qq(\n<h$level id="$id">).$self->line($line,1).qq(</h$level>\n);
}

sub hr {
	my $self = shift;
	my ($line) = @_;
	return qq(<hr />\n);
}

sub pre {
	my $self = shift;
	my ($line) = @_;

	$line =~ s/^ //;
	my $html = qq(<pre>).cdata($line).qq(\n);
	while (! $self->eof) {
		$line = $self->next_line;
		last if ($line !~ /^\s/);
		$line = $self->read_line;

		$line =~ s/^ //;
		$html .= cdata($line).qq(\n);
	}
	chomp $html;
	return $html.qq(</pre>\n);
}

sub li {
	my $self = shift;
	my ($line, $uo, $level) = @_;

	$level++;
	my $html = qq(<$uo>\n);
	$line =~ /^(\-+)\s*(.*)$/	if ($uo eq 'ul');
	$line =~ /^(\++)\s*(.*)$/	if ($uo eq 'ol');
	$html .= ($level < length($1))
				? qq(<li style="list-style-type: none;">)
					.$self->li($line, $uo, $level)
				: qq(<li>).$self->line($2);
	while (! $self->eof) {
		$line = $self->next_line;
		last if ($line =~ /^\-*$/);
		last if ($line =~ /^\/\*$/);
		last if ($line =~ /^\/\//);
		last if ($line =~ /^[\~\<\=\>\!\*\s\:\|\#]/);

		if ($line =~ /^(\-+)\s*(.*)$/) {
			last if ($uo eq 'ul' && length($1) < $level);
			last if ($uo eq 'ol' && length($1) <= $level);
			$line = $self->read_line;
			if ($level < length($1)) {
				$html .= $self->li($line, 'ul', $level);
			}
			else {
				chomp $html;
				$html .= qq(</li>\n<li>).$self->line($2);
			}
		}
		elsif ($line =~ /^(\++)\s*(.*)$/) {
			last if ($uo eq 'ol' && length($1) < $level);
			last if ($uo eq 'ul' && length($1) <= $level);
			$line = $self->read_line;
			if ($level < length($1)) {
				$html .= $self->li($line, 'ol', $level);
			}
			else {
				chomp $html;
				$html .= qq(</li>\n<li>).$self->line($2);
			}
		}
		else {
			$line = $self->read_line;
			$html .= $self->line($line);
		}
	}
	chomp $html;
	return $html.qq(</li>\n</$uo>\n);
}

sub dl {
	my $self = shift;
	my ($line) = @_;

	my $html = qq(<dl>\n);
	$line =~ s/^://;
	$html .= qq(<dt>).$self->line($line,1).qq(</dt>\n);
	while (! $self->eof) {
		$line = $self->next_line;
		last if ($line =~ /^$/);
		last if ($line =~ /^\/\*$/);
		last if ($line =~ /^\/\//);
		last if ($line =~ /^[\~\<\=\>\!\*\s\-\+\|\#]/);
		$line = $self->read_line;

		if ($line =~ s/^://) {
			if (length($line)) {
				$html .= qq(<dt>).$self->line($line,1).qq(</dt>\n);
			}
		}
		else {
			$html .= $self->dd($line);
		}
	}
	return $html.qq(</dl>\n);
}

sub dd {
	my $self = shift;
	my ($line) = @_;

	my $html = qq(<dd>).$self->line($line);
	while (! $self->eof) {
		$line = $self->next_line;
		last if ($line =~ /^$/);
		last if ($line =~ /^\/\*$/);
		last if ($line =~ /^\/\//);
		last if ($line =~ /^[\~\<\=\>\!\*\s\-\+\:\|\#]/);
		$line = $self->read_line;

		$html .= $self->line($line);
	}
	chomp $html;
	return $html.qq(</dd>\n);
}

sub table {
	my $self = shift;
	my ($line) = @_;

	my $table = [];

	$line =~ s/^\|//;
	push(@{$table}, [ split(/\|/, $line) ]);
	while (! $self->eof) {
		$line = $self->next_line;
		last if ($line !~ /^\|/);
		$line = $self->read_line;

		$line =~ s/^\|//;
		push(@{$table}, [ split(/\|/, $line) ]);
	}

	my $html = qq(<table>\n);
	while (@{$table}) {
		my $tr = shift @{$table};
		my $col = 0;

		$html .= qq(<tr>);
		while (@{$tr}) {
			$col++;
			my $cell = shift @{$tr};
			next if ($cell =~ /^[\-\^]/);

			my $span = '';
			my $colspan = 1;
			foreach (@{$tr}) { last if (! /^-/); $colspan++; }
			$span .= qq( colspan="$colspan")	if ($colspan > 1);

			my $rowspan = 1;
			foreach (@{$table})
							{ last if ($_->[$col -1] !~ /^\^/); $rowspan++; }
			$span .= qq( rowspan="$rowspan")	if ($rowspan > 1);

			my $tag = ($cell =~ s/^~//) ? 'th' : 'td';

			my $align = '';
			if    ($cell =~ s/^=//) {
				$align = qq( style="text-align: center;");
			}
			elsif ($cell =~ s/^>//) {
				$align = qq( style="text-align: right;");
			}
			elsif ($cell =~ s/^<//) {
				$align = qq( style="text-align: left;");
			}

			$html .= qq(<$tag$span$align>).$self->line($cell,1).qq(</$tag>);
		}
		$html .= qq(</tr>\n);
	}
	return $html.qq(</table>\n);
}

sub blockquote {
	my $self = shift;
	my $html = qq(<blockquote>\n);
	$html .= $self->block('<<');
	return $html.qq(</blockquote>\n);
}

sub block_pre {
	my $self = shift;
	my $html = qq(<pre>);
	$html .= $self->np_block('|<');
	chomp $html;
	return $html.qq(</pre>\n);
}

sub line {
	my $self = shift;
	my ($line, $no_eol) = @_;

	$line =~ s/'''([^'].*?)'''/&_(strong){$1};/g;
	$line =~ s/''([^'].*?)''/&_(em){$1};/g;
	$line =~ s/%%([^%].*?)%%/&_(del){$1};/g;
	$line =~ s/\[\[([^\[].*?)\]\]/$self->braket($1)/eg;
	$line =~ s/\{\{([^\{].*?)\}\}/&__($1);/g;

	my $eol = $no_eol ? '' : ($line =~ s/(~)$// ? qq(<br />\n) : qq(\n));
	return $self->inline($line).$eol;
}

sub braket {
	my $self = shift;
	$_ = shift;

#	return qq(&_(a href="$2"){$1};)		if (/^(.*)[:>]($link_regex)$/);
	return qq(&_(a href="$2"){$1};)		if (/^(.*)>(.*)$/);
	return qq(&_(a href="$_"){$_};);
}

sub inline {
	my $self = shift;
	my ($inline) = @_;

	my @elem = $inline =~ /((?:(?!$module_regex).)*)($module_regex)?/g;
	my $html;
	while (@elem) {
		$html .= $self->text(shift @elem);
		$html .= $self->module(shift @elem);
	}
	return $html;
}

sub module {
	my $self = shift;
	my ($module) = @_;

	return '' if (! defined $module);
	
	my ($name, $param, $value)
			= $module =~ /^&(\w+)(?:\((.*?)\))?(?:{(.*)})?;$/;
	$value = $self->inline($value)		if (defined $value);

	return $self->tag($param, $value)			if ($name eq '_');
	return cdata($param)						if ($name eq '__');

	my $html = $self->{module}->dispatch('inline', $name, $param, $value);
	return defined $html ? $html : $self->text($module);
}

sub block_module {
	my $self = shift;
	my ($module) = @_;

	my ($name, $param, $eof)
			= $module =~ /^#(\w+)(?:\((.*?)\))?(?:<<(.*))?$/;
	return $self->p($module)		if (! defined $name);

	my $value = $self->block($eof)	if (defined $eof);

	return $self->tag($param, $value, 1)		if ($name eq '_');
	return $self->contents						if ($name eq 'contents');

	my $html = $self->{module}->dispatch('block', $name, $param, $value);
	return defined $html ? $html : $self->p($module);
}

sub text {
	my $self = shift;
	my ($text) = @_;

	return ''	if (! defined $text);

	my @elem = $text =~ /((?:(?!$link_regex).)*)($link_regex)?/g;
	my $html;
	while (@elem) {
		$html .= cdata(shift @elem);
		my $link = shift @elem		or next;
		my $value = cdata($link);
		my $param = ($link =~ /:/) ? $value : cdata("mailto:$link");
		$html .= qq(<a href="$param">$value</a>);
	}
	return $html;
}

sub tag {
	my $self = shift;
	my ($param, $value, $block) = @_;

	if (defined $param && $param =~ /^(\w+)((?:\s+\w+="[^"]*")*)$/) {
		my $name  = $1;
		my $param = $2;
		$param =~ s/"([^"]*)"/qq(").cdata($1).qq(")/eg;
		return	($block)
					?  (defined $value)
							? qq(<$name$param>\n$value</$name>\n)
							: qq(<$name$param />\n)
					:  (defined $value)
							? qq(<$name$param>$value</$name>)
							: qq(<$name$param />);
	}
	return (defined $value) ? $value : '';
}

sub contents {
	my $self = shift;
	my $c_toc = @{$self->{toc}};

	my $value = $self->block;

	my @toc = @{$self->{toc}};
	splice(@toc, 0, $c_toc);

	my $html = qq(<div class="contents">\n)
			 . $self->parse(join("\n", @toc))
			 . qq(</div>\n);
	return $html.$value;
}

1;
