# $Id: Util.pm,v 1.2 2006/12/29 17:59:59 koba Exp $
#	Copyright(c) 2006 Satoshi Kobayashi, All rights reserved.

package XiuMai::Util;

use strict;
use Exporter ();
use vars qw($VERSION @ISA @EXPORT_OK);

$VERSION = do { my @r = (q$Revision: 1.2 $ =~ /\d+/g);
				sprintf("%d." . "%02d" x $#r, @r) };

@ISA = qw(Exporter);

@EXPORT_OK = qw(
	cdata url_encode url_decode
	date_str size_str
	rfc1123_date
	canonpath
	randstr uniq_id
);

my $serial;

sub cdata {
	my ($str) = @_;
	return ''	if (! defined $str);
	$str =~ s/&/&amp;/g;
	$str =~ s/"/&quot;/g;
	$str =~ s/</&lt;/g;
	$str =~ s/>/&gt;/g;
	$str =~ /^(.*)$/s		and $str = $1;
	return $str;
}

sub url_encode {
	my ($url) = @_;
	$url =~ s/([^\w\-\*\.])/sprintf("%%%02X", unpack('C', $1))/ge;
	return $url;
}

sub url_decode {
	my ($str) = @_;
	$str =~ s/%([\da-fA-F]{2})/pack('H2', $1)/ge;
	return $str;
}

sub date_str {
	my ($time) = @_;

	my $now = time;

	$time = $now	if (! defined $time);
	my ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime($time);

	if (abs($now - $time) < 60*60*12) {
		return sprintf("%02d:%02d", $hour, $min);
	}
	elsif (abs($now - $time) < 60*60*24*365/2) {
		return sprintf("%d/%d %02d:%02d", $mon+1, $mday, $hour, $min);
	}
	else {
		return sprintf("%d/%02d/%02d", $year+1900, $mon+1, $mday);
	}
}

sub size_str {
	my ($size) = @_;

	return '-'	if (! defined $size);

	my $size_str;
	my @unit = ('%d','%dK','%.1fM','%.1fG','%.1fT');
	while (my $unit = shift @unit) {
		$size_str = sprintf $unit, $size;
		last if ($size < 1024);
		$size = $size / 1024;
	}
	return $size_str;
}

sub rfc1123_date {
	my ($time) = @_;

	$time = time	if (! defined $time);

	my @wday = qw(Sun Mon Tue Wed Thu Fri Sat);
	my @mon  = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

	my ($sec, $min, $hour, $mday, $mon, $year, $wday) = gmtime($time);

	return sprintf("%s, %02d %s %d %02d:%02d:%02d GMT",
					$wday[$wday], $mday, $mon[$mon], $year+1900,
					$hour, $min, $sec);
}

sub canonpath {
	my ($path) = @_;
	$path =~ s|/+|/|g;
	$path =~ s|^/||		or return '';
	my $dir = $path =~ s|/$||;
	my @path;
	foreach (split(m|/|, $path)) {
		next if ($_ eq '.');
		if ($_ eq '..') { pop(@path);		}
		else			{ push(@path, $_);	}
	}
	return '/' . join('/', @path) . (@path && $dir ? '/' : '');
}

sub randstr {
	my ($len, $base_str) = @_;

	$base_str = '0123456789'
			  . 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'	if (! $base_str);
	my $str;
	for (1..$len) {
		$str .= substr($base_str, int(rand() * (length $base_str)), 1);
	}
	return $str;
}

sub uniq_id {
	my ($short) = @_;

	if ($short) {
		return sprintf("%08X%04X%04X", time, $$, $serial++ % 65536);
	}
	else {
		my ($sec, $min, $hour, $day, $mon, $year) = localtime;
		return sprintf("%04d%02d%02d%02d%02d%02d-%04X%04X",
						$year + 1900, $mon + 1, $day, $hour, $min, $sec,
						$$, $serial++ % 65536);
	}
}

sub dump {
	my ($ref, $level, $visit) = @_;

	my $dump;

	$level++;
	my $refstr = (defined $ref) ? "$ref" : "( undef )";
	$dump .= $refstr;

	if ($visit->{$refstr}) {
		return $dump .= " -> ( **VISITED** )\n";
	}
	$dump .= "\n";

	if ($refstr =~ /HASH\(/) {
		$visit->{$refstr} = 1;
		foreach my $key (sort keys %$ref) {
			$dump .= ('    ' x $level).qq({ $key } : )
				   . XiuMai::Util::dump($ref->{$key}, $level, $visit);
		}
	}
	elsif ($refstr =~ /ARRAY\(/) {
		$visit->{$refstr} = 1;
		for (my $i = 0; $i < @$ref; $i++) {
			$dump .= ('    ' x $level).qq([ $i ] : )
				   . XiuMai::Util::dump($ref->[$i], $level, $visit);
		}
	}

	return $dump;
}

1;
